@@ -25,11 +25,11 @@ from IPython.core.displaypub import DisplayPublisher
 from IPython.core.error import UsageError
 from IPython.core.interactiveshell import InteractiveShell, InteractiveShellABC
 from IPython.core.magic import Magics, line_magic, magics_class
-from IPython.core.magics import CodeMagics, MacroToEdit
+from IPython.core.magics import CodeMagics, MacroToEdit  # type:ignore[attr-defined]
 from IPython.core.usage import default_banner
-from IPython.display import Javascript, display
+from IPython.display import Javascript, display  # type:ignore[attr-defined]
 from IPython.utils import openpy
-from IPython.utils.process import arg_split, system
+from IPython.utils.process import arg_split, system  # type:ignore[attr-defined]
 from jupyter_client.session import Session, extract_header
 from jupyter_core.paths import jupyter_runtime_dir
 from traitlets import Any, CBool, CBytes, Dict, Instance, Type, default, observe
@@ -144,11 +144,17 @@ class ZMQDisplayPublisher(DisplayPublisher):
         """
         content = dict(wait=wait)
         self._flush_streams()
+        msg = self.session.msg("clear_output", json_clean(content), parent=self.parent_header)
+
+        # see publish() for details on how this works
+        for hook in self._hooks:
+            msg = hook(msg)
+            if msg is None:
+                return
+
         self.session.send(
             self.pub_socket,
-            "clear_output",
-            content,
-            parent=self.parent_header,
+            msg,
             ident=self.topic,
         )
 
@@ -296,6 +302,7 @@ class KernelMagics(Magics):
         filename = os.path.abspath(filename)
 
         payload = {"source": "edit_magic", "filename": filename, "line_number": lineno}
+        assert self.shell is not None
         self.shell.payload_manager.write_payload(payload)
 
     # A few magics that are adapted to the specifics of using pexpect and a
@@ -304,6 +311,7 @@ class KernelMagics(Magics):
     @line_magic
     def clear(self, arg_s):
         """Clear the terminal."""
+        assert self.shell is not None
         if os.name == "posix":
             self.shell.system("clear")
         else:
@@ -321,9 +329,11 @@ class KernelMagics(Magics):
 
         Files ending in .py are syntax-highlighted."""
         if not arg_s:
-            raise UsageError("Missing filename.")
+            msg = "Missing filename."
+            raise UsageError(msg)
 
         if arg_s.endswith(".py"):
+            assert self.shell is not None
             cont = self.shell.pycolorize(openpy.read_py_file(arg_s, skip_encoding_cookie=False))
         else:
             with open(arg_s) as fid:
@@ -338,6 +348,7 @@ class KernelMagics(Magics):
         @line_magic
         def man(self, arg_s):
             """Find the man page for the given command and display in pager."""
+            assert self.shell is not None
             page.page(self.shell.getoutput("man %s | col -b" % arg_s, split=False))
 
     @line_magic
@@ -358,7 +369,7 @@ class KernelMagics(Magics):
             connection_file = get_connection_file()
             info = get_connection_info(unpack=False)
         except Exception as e:
-            warnings.warn("Could not get connection info: %r" % e)
+            warnings.warn("Could not get connection info: %r" % e, stacklevel=2)
             return
 
         # if it's in the default dir, truncate to basename
@@ -394,7 +405,7 @@ class KernelMagics(Magics):
         try:
             connect_qtconsole(argv=arg_split(arg_s, os.name == "posix"))
         except Exception as e:
-            warnings.warn("Could not start qtconsole: %r" % e)
+            warnings.warn("Could not start qtconsole: %r" % e, stacklevel=2)
             return
 
     @line_magic
@@ -430,7 +441,7 @@ class ZMQInteractiveShell(InteractiveShell):
 
     displayhook_class = Type(ZMQShellDisplayHook)
     display_pub_class = Type(ZMQDisplayPublisher)
-    data_pub_class = Any()
+    data_pub_class = Any()  # type:ignore[assignment]
     kernel = Any()
     parent_header = Any()
 
@@ -511,7 +522,7 @@ class ZMQInteractiveShell(InteractiveShell):
                 stacklevel=2,
             )
 
-            self._data_pub = self.data_pub_class(parent=self)
+            self._data_pub = self.data_pub_class(parent=self)  # type:ignore[has-type]
             self._data_pub.session = self.display_pub.session
             self._data_pub.pub_socket = self.display_pub.pub_socket
         return self._data_pub
@@ -624,7 +635,8 @@ class ZMQInteractiveShell(InteractiveShell):
             # pexpect or pipes to read from.  Users can always just call
             # os.system() or use ip.system=ip.system_raw
             # if they really want a background process.
-            raise OSError("Background processes not supported.")
+            msg = "Background processes not supported."
+            raise OSError(msg)
 
         # we explicitly do NOT return the subprocess status code, because
         # a non-None value would trigger :func:`sys.displayhook` calls.
